###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
# Copyright (c) 2006 Mehdi Abaakouk
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###
#
# Inspired on rhymthbox metadata C code
# rhythmbox/metadata/rb-metadata-gst.c
##


""" Gstreamer tag reader is for remote file only """

import gst
import gobject

class GstTagReadingError(Exception): 
    pass

def read(song):
    IDS = {
        "title": "title",
        "genre": "genre",
        "artist": "artist",
        "album": "album",
        "bitrate": "#bitrate",
        'track-number':"#track"
    }
    
    is_eos = False
    is_tagged = False
    def unknown_type(*param):
        print "Gst decoder: type inconnu"
        raise GstTagReadingError
    
    def eos(pipeline,raise_error=False):
        state_ret = pipeline.set_state(gst.STATE_NULL)
        if state_ret != gst.STATE_CHANGE_SUCCESS:
            print "Failed change to null"
        is_eos = True
        #print "EOS"
            
        
    def message(bus,message,pipeline,song):
        if message.type == gst.MESSAGE_EOS:
            eos(pipeline)

        elif message.type == gst.MESSAGE_TAG:
            taglist = message.parse_tag()
            for key in taglist.keys():
                if IDS.has_key(key): 
                    if key=="bitrate":
                        value = int(taglist[key]/100)
                    elif isinstance(taglist[key],long):
                        value = int(taglist[key])   
                    else:
                        value = taglist[key]
                    song[IDS[key]] = value
                    #print key,":",value
            song["tagged"] = True
            is_tagged = True

        elif message.type == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            print "Decoder error: %s" % err,debug
            eos(pipeline)
            raise GstTagReadingError
        
    try: pipeline = gst.parse_launch ("gnomevfssrc location="+song.get("uri")+" ! decodebin name=decoder ! fakesink");
    except gobject.GError : 
        print "W:GstTag:Failed to build pipeline to read metadata of",song.get("uri")
        raise GstTagReadingError
    #pipeline = gst.parse_launch ("gnomevfssrc location="+song.get("uri")+" ! decodebin name=decoder ! fakesink");
    decoder = pipeline.get_by_name("decoder")
    decoder.connect("unknown-type",unknown_type)
    bus = pipeline.get_bus()
    bus.connect('message', message,pipeline,song)
    bus.add_signal_watch()
    
    state_ret = pipeline.set_state(gst.STATE_PAUSED);
    timeout = 10
    state = None
    while state_ret == gst.STATE_CHANGE_ASYNC and not is_eos and timeout > 0:
        state_ret,state,pending_state = pipeline.get_state(1 * gst.SECOND);
        timeout -= 1
        
    if state_ret != gst.STATE_CHANGE_SUCCESS:
        print "Failed change to paused"
        eos(pipeline)
        raise GstTagReadingError
    else:
        if not is_tagged:
            bus.poll(gst.MESSAGE_TAG,10 * gst.SECOND)
        try:
            query = gst.query_new_duration(gst.FORMAT_TIME)
            if pipeline.query(query):
                total = query.parse_duration()[1]
            else: total = 0
        except gst.QueryError: total = 0
        total //= gst.MSECOND
        #print "duration",total
        song["#duration"] = total
        if not song["tagged"]:
            print "W:GstTag: Media found but no tag found"
        
        eos(pipeline)
        
    return True
        
        
    
